use std::collections::HashMap;

use enum_map::EnumMap;
use rgb::RGB8;
use serde::Deserialize;

use crate::model;
use crate::model::image_filter::ImageFilter;
use crate::model::rgb::DeserializableRGB8;
use crate::model::ButtonPosition;

#[derive(Debug, Deserialize)]
pub struct File {
    pub icon_packs: Vec<IconPack>,
    #[serde(default = "inactive_button_color_default")]
    pub inactive_button_color: DeserializableRGB8,
    #[serde(default = "active_button_color_default")]
    pub active_button_color: DeserializableRGB8,
    pub buttons: HashMap<ButtonPosition, ButtonConfig>, // EnumMap
    pub initial: InitialConfig,
}

#[derive(Debug)]
pub struct WithFallbackId<T> {
    pub fallback_id: String,
    pub inner: T,
}

#[derive(Debug)]
pub struct Config {
    pub key_pages_by_id: HashMap<String, model::key_page::Page>,
    pub knob_pages_by_id: HashMap<String, model::knob_page::Page>,
    pub icon_packs: Vec<IconPack>,
    pub inactive_button_color: DeserializableRGB8,
    pub active_button_color: DeserializableRGB8,
    pub buttons: EnumMap<ButtonPosition, ButtonConfig>,
    pub initial: InitialConfig,
}

fn inactive_button_color_default() -> DeserializableRGB8 {
    DeserializableRGB8(RGB8::new(128, 128, 128))
}

fn active_button_color_default() -> DeserializableRGB8 {
    DeserializableRGB8(RGB8::new(0, 255, 0))
}

#[derive(Debug, Deserialize, Default)]
pub struct ButtonConfig {
    pub key_page: Option<String>,
    pub knob_page: Option<String>,
}

#[derive(Debug, Deserialize)]
pub struct InitialConfig {
    pub key_page: String,
    pub knob_page: String,
}

#[derive(Debug, Deserialize)]
pub struct IconPack {
    pub id: String,
    pub path: String,
    pub global_filter: Option<ImageFilter>,
}
