use std::collections::HashMap;
use std::num::NonZeroU8;

use regex::Regex;
use serde::Deserialize;

use crate::model::icon_descriptor::IconDescriptor;
use crate::model::rgb::DeserializableRGB8WithOptionalAlpha;

#[derive(Debug, Deserialize)]
pub struct Config {
    #[serde(with = "serde_regex")]
    pub regex: Regex,
    #[serde(default)]
    pub direction: Direction,
    #[serde(default)]
    pub disable_press_to_mute: bool,
    #[serde(default)]
    pub disable_press_to_unmute: bool,
    #[serde(default)]
    pub muted_turn_action: MutedTurnAction,
    #[serde(default)]
    pub label: HashMap<State, String>,
    #[serde(default)]
    pub icon: HashMap<State, IconDescriptor>,
    pub circle_indicator: Option<CircleIndicatorConfig>,
    pub bar_indicator: Option<BarIndicatorConfig>,
}

#[derive(Debug, Default, Eq, PartialEq, Deserialize)]
#[serde(rename_all = "kebab-case")]
pub enum MutedTurnAction {
    #[default]
    Ignore,
    UnmuteAtZero,
    UnmuteAndRestore,
}

#[derive(Debug, Default, Eq, PartialEq, Deserialize)]
#[serde(rename_all = "kebab-case")]
pub enum Direction {
    #[default]
    Output,
    Input,
}

#[derive(Debug, Eq, PartialEq, Hash, Deserialize)]
#[serde(rename_all = "kebab-case")]
pub enum State {
    Inactive,
    Active,
    Muted,
}

#[derive(Debug, Deserialize)]
pub struct CircleIndicatorConfig {
    pub color: DeserializableRGB8WithOptionalAlpha,
    pub width: NonZeroU8,
    pub radius: u8,
}

#[derive(Debug, Deserialize)]
pub struct BarIndicatorConfig {
    pub color: DeserializableRGB8WithOptionalAlpha,
}
