use std::collections::HashMap;

use enum_map::EnumMap;
use serde::Deserialize;

use crate::model::icon_descriptor::IconDescriptor;
use crate::model::rgb::DeserializableRGB8WithOptionalAlpha;
use crate::model::{knob_modes, KnobPosition};

#[derive(Debug, Deserialize)]
pub struct File {
    pub id: Option<String>,
    pub knobs: HashMap<KnobPosition, Knob>,
}

#[derive(Debug)]
pub struct Page {
    pub id: String,
    pub knobs: EnumMap<KnobPosition, Knob>,
}

#[derive(Debug, Default, Deserialize)]
pub struct Knob {
    #[serde(default)]
    pub label: String,
    #[serde(default)]
    pub icon: IconDescriptor,
    #[serde(default)]
    pub indicator: KnobIndicators,
    #[serde(default)]
    pub mode: KnobModes,
}

#[derive(Debug, Default, Deserialize)]
pub struct KnobIndicators {
    pub bar: Option<KnobIndicatorBarConfig>,
    pub circle: Option<KnobIndicatorCircleConfig>,
}

#[derive(Debug, Deserialize)]
pub struct KnobIndicatorBarConfig {
    pub color: DeserializableRGB8WithOptionalAlpha,
}

#[derive(Debug, Deserialize)]
pub struct KnobIndicatorCircleConfig {
    pub color: DeserializableRGB8WithOptionalAlpha,
    pub width: u8,
    pub radius: u8,
}

#[derive(Debug, Default, Deserialize)]
pub struct KnobModes {
    pub audio_volume: Option<knob_modes::audio_volume::Config>,
}
